import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.*;
import javax.swing.SpringLayout;

import com.mindfusion.diagramming.*;

public class AnnealLayoutFrame extends JFrame{

	public AnnealLayoutFrame()
	{
		super("MindFusion.Diagramming Sample: Annealing Layout");
		initFrame();
	}
	
	public void applyLayout()
	{
		int initIter;
		int initStages;
		double initTemp;
		double initTempScale;
		double arrowLengthFactor;
		double crossingArrowsCost;
		double distributionFactor;
		double nodeArrowDistFactor;
		double boundaryFactor;
		
		String errorValue = "";

		// parse cost fields
		try
		{
			errorValue = iterationTxF.getText();
			initIter = Integer.valueOf(iterationTxF.getText());
			errorValue = stageTxF.getText();
			initStages = Integer.valueOf(stageTxF.getText());
			errorValue = initialTxF.getText();
			initTemp = Double.valueOf(initialTxF.getText());
			errorValue = tempScaleTxF.getText();
			initTempScale = Double.valueOf(tempScaleTxF.getText());
			
			errorValue = arrowLengthFactorTxF.getText();
			arrowLengthFactor = Double.valueOf(arrowLengthFactorTxF.getText());
			errorValue = crossingArrowsCostTxF.getText();
			crossingArrowsCost = Double.valueOf(crossingArrowsCostTxF.getText());
			errorValue = distributionFactorTxF.getText();
			distributionFactor = Double.valueOf(distributionFactorTxF.getText());
			errorValue = nodeArrowDistFactorTxF.getText();
			nodeArrowDistFactor = Double.valueOf(nodeArrowDistFactorTxF.getText());
			errorValue = boundaryFactorTxF.getText();
			boundaryFactor = Double.valueOf(boundaryFactorTxF.getText());
			
		}
		catch (NumberFormatException exp)
		{
			JOptionPane.showMessageDialog(
				AnnealLayoutFrame.this, 
				"Error! Incorrect input: " + errorValue);
			return;
		}

		// assign cost values to AnnealLayout
		AnnealLayout layout = new AnnealLayout();
		layout.setIterationsPerStage(initIter); 
		layout.setStages(initStages); 
		layout.setInitialTemperature(initTemp);
		layout.setTemperatureScale(initTempScale); 
		layout.setLinkLengthFactor(arrowLengthFactor);
		layout.setCrossingLinksCost(crossingArrowsCost);
		layout.setDistributionFactor(distributionFactor);
		layout.setNodeLinkDistFactor(nodeArrowDistFactor);
		layout.setBoundaryFactor(boundaryFactor);
		layout.setPrecision(0.995);
		layout.setNodeLinkCrossingCost(200000);

		layout.arrange(diagram);
		diagram.resizeToFitItems(10);
	}
	
	public void onLoadBtn()
	{
		JFileChooser chooser = new JFileChooser();
		
		if (chooser.showOpenDialog(AnnealLayoutFrame.this) == JFileChooser.APPROVE_OPTION)
		{
			try 
			{
				diagram.loadFrom(chooser.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException exp) 
			{
				JOptionPane.showMessageDialog(AnnealLayoutFrame.this, exp.getMessage());
			}
			catch (IOException exp)
			{
				JOptionPane.showMessageDialog(AnnealLayoutFrame.this, exp.getMessage());
			}
		}
	}
	
	public void onSaveBtn()
	{
		JFileChooser chooser = new JFileChooser();
		
		if (chooser.showSaveDialog(AnnealLayoutFrame.this) == JFileChooser.APPROVE_OPTION)
		{
			try 
			{
				diagram.saveTo(chooser.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException exp) 
			{
				JOptionPane.showMessageDialog(AnnealLayoutFrame.this, exp.getMessage());
			}
			catch (IOException exp)
			{
				JOptionPane.showMessageDialog(AnnealLayoutFrame.this, exp.getMessage());
			}
		}
	}
	
	public void onImportBtn()
	{
		JFileChooser chooser = new JFileChooser();
		chooser.setDialogTitle("Import");
		if (chooser.showOpenDialog(AnnealLayoutFrame.this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				diagram.loadFromXml(chooser.getSelectedFile().getAbsolutePath());
			} catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (XmlException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void onExportBtn()
	{
		JFileChooser chooser = new JFileChooser();
		chooser.setDialogTitle("Export");
		if (chooser.showSaveDialog(AnnealLayoutFrame.this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				diagram.saveToXml(chooser.getSelectedFile().getAbsolutePath());
			} catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (XmlException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
		}
	}

	private void initFrame()
	{
		diagram = new Diagram();
		diagramView = new DiagramView();
		diagramView.setDiagram(diagram);
		
		diagViewScrP = new JScrollPane(diagramView);
		layoutBtn = new JButton();
		loadBtn = new JButton();
		saveBtn = new JButton();
		importBtn = new JButton();
		exportBtn = new JButton();

		textTxA = new JTextArea();
		textScrP = new JScrollPane(textTxA);

		optionPnl = new JPanel();

		iterationLb = new JLabel();
		stageLb = new JLabel();
		initialLb = new JLabel();
		tempScaleLb = new JLabel();
		arrowLengthFactorLb = new JLabel();
		crossingArrowsCostLb = new JLabel();
		distributionFactorLb = new JLabel();
		nodeArrowDistFactorLb = new JLabel();
		boundaryFactorLb = new JLabel();
		
		iterationTxF = new JTextField();
		stageTxF = new JTextField();
		initialTxF = new JTextField();
		tempScaleTxF = new JTextField();
		arrowLengthFactorTxF = new JTextField();
		crossingArrowsCostTxF = new JTextField();
		distributionFactorTxF = new JTextField();
		nodeArrowDistFactorTxF = new JTextField();
		boundaryFactorTxF = new JTextField();
		
		// diag
		diagram.setBackBrush(new com.mindfusion.diagramming.SolidBrush(new Color(242,242,242)));
		diagram.getShapeNodeStyle().setBrush(
			new GradientBrush(Color.white, new Color(135, 206, 250), 5));
		diagram.setDefaultShape(Shape.fromId("Ellipse"));
		diagram.setBounds(new Rectangle2D.Float(0, 0, 350, 220));

		// layoutBtn
		layoutBtn.setName("layoutBtn");
		layoutBtn.setText("Layout");
		layoutBtn.setBounds(5, 5, 100, 20);
		layoutBtn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				applyLayout();
			}
		});

		// loadBtn
		loadBtn.setName("loadBtn");
		loadBtn.setText("Load");
		loadBtn.setBounds(5, 30, 100, 20);
		loadBtn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				onLoadBtn();
			}
		});

		// saveBtn
		saveBtn.setName("saveBtn");
		saveBtn.setText("Save");
		saveBtn.setBounds(5, 55, 100, 20);
		saveBtn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				onSaveBtn();
			}
		});

		// importBtn
		importBtn.setName("importBtn");
		importBtn.setText("Import XML");
		importBtn.setBounds(5, 80, 100, 20);
		importBtn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				onImportBtn();
			}
		});

		// importBtn
		exportBtn.setName("exportBtn");
		exportBtn.setText("Export XML");
		exportBtn.setBounds(5, 105, 100, 20);
		exportBtn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				onExportBtn();
			}
		});
		
		// optionPnl
		optionPnl.setName("optionPnl");
		optionPnl.setLayout(null);
		optionPnl.setBackground(Color.white);
		optionPnl.add(layoutBtn);
		optionPnl.add(loadBtn);
		optionPnl.add(saveBtn);
		optionPnl.add(importBtn);
		optionPnl.add(exportBtn);
		optionPnl.add(iterationLb);
		optionPnl.add(stageLb);
		optionPnl.add(initialLb);
		optionPnl.add(tempScaleLb);
		optionPnl.add(arrowLengthFactorLb);
		optionPnl.add(crossingArrowsCostLb);
		optionPnl.add(distributionFactorLb);
		optionPnl.add(nodeArrowDistFactorLb);
		optionPnl.add(boundaryFactorLb);
		optionPnl.add(iterationTxF);
		optionPnl.add(stageTxF);
		optionPnl.add(initialTxF);
		optionPnl.add(tempScaleTxF);
		optionPnl.add(arrowLengthFactorTxF);
		optionPnl.add(crossingArrowsCostTxF);
		optionPnl.add(distributionFactorTxF);
		optionPnl.add(nodeArrowDistFactorTxF);
		optionPnl.add(boundaryFactorTxF);

		// iterationLb
		iterationLb.setName("iterationLb");
		iterationLb.setText("Iterations Per Stage");
		iterationLb.setBounds(110, 5, 120, 20);

		// stageLb
		stageLb.setName("stageLb");
		stageLb.setText("Stages");
		stageLb.setBounds(110, 30, 120, 20);

		// initialLb
		initialLb.setName("initialLb");
		initialLb.setText("Initial Temperature");
		initialLb.setBounds(110, 55, 120, 20);

		// tempScaleLb
		tempScaleLb.setName("tempScaleLb");
		tempScaleLb.setText("Temperature Scale");
		tempScaleLb.setBounds(110, 80, 120, 20);

		// arrowLengthFactorLb
		arrowLengthFactorLb.setName("arrowLengthFactorLb");
		arrowLengthFactorLb.setText("Arrow Length Factor");
		arrowLengthFactorLb.setBounds(315, 5, 135, 20);
		
		// crossingArrowsCostLb
		crossingArrowsCostLb.setName("crossingArrowsCostLb");
		crossingArrowsCostLb.setText("Crossing Arrows Cost");
		crossingArrowsCostLb.setBounds(315, 30, 135, 20);

		// distributionFactorLb
		distributionFactorLb.setName("distributionFactorLb");
		distributionFactorLb.setText("Distribution Factor");
		distributionFactorLb.setBounds(315, 55, 135, 20);
		
		// nodeArrowDistFactorLb
		nodeArrowDistFactorLb.setName("nodeArrowDistFactorLb");
		nodeArrowDistFactorLb.setText("Node Arrow Dist Factor");
		nodeArrowDistFactorLb.setBounds(315, 80, 135, 20);

		// boundaryFactorLb
		boundaryFactorLb.setName("boundaryFactorLb");
		boundaryFactorLb.setText("Boundary Factor");
		boundaryFactorLb.setBounds(315, 105, 135, 20);
		
		AnnealLayout _layout = new AnnealLayout();
		// iterationTxF
		iterationTxF.setName("iterationTxF");
		iterationTxF.setText(String.valueOf(_layout.getIterationsPerStage()));
		iterationTxF.setBounds(235, 5, 60, 20);

		// stageTxF
		stageTxF.setName("stageTxF");
		stageTxF.setText(String.valueOf(_layout.getStages()));
		stageTxF.setBounds(235, 30, 60, 20);

		// initialTxF
		initialTxF.setName("initialTxF");
		initialTxF.setText(String.valueOf(_layout.getInitialTemperature()));
		initialTxF.setBounds(235, 55, 60, 20);

		// tempScaleTxF
		tempScaleTxF.setName("tempScaleTxF");
		tempScaleTxF.setText(String.valueOf(_layout.getTemperatureScale()));
		tempScaleTxF.setBounds(235, 80, 60, 20);

		// arrowLengthFactorTxF
		arrowLengthFactorTxF.setName("arrowLengthFactorTxF");
		arrowLengthFactorTxF.setText(String.valueOf(_layout.getLinkLengthFactor()));
		arrowLengthFactorTxF.setBounds(455, 5, 60, 20);
		
		// crossingArrowsCostTxF
		crossingArrowsCostTxF.setName("crossingArrowsCostTxF");
		crossingArrowsCostTxF.setText(String.valueOf(_layout.getCrossingLinksCost()));
		crossingArrowsCostTxF.setBounds(455, 30, 60, 20);
		
		// distributionFactorTxF
		distributionFactorTxF.setName("distributionFactorTxF");
		distributionFactorTxF.setText(String.valueOf(_layout.getDistributionFactor()));
		distributionFactorTxF.setBounds(455, 55, 60, 20);
		
		// nodeArrowDistFactorTxF
		nodeArrowDistFactorTxF.setName("nodeArrowDistFactorTxF");
		nodeArrowDistFactorTxF.setText(String.valueOf(_layout.getNodeLinkDistFactor()));
		nodeArrowDistFactorTxF.setBounds(455, 80, 60, 20);

		// boundaryFactorTxF
		boundaryFactorTxF.setName("boundaryFactorTxF");
		boundaryFactorTxF.setText(String.valueOf(_layout.getBoundaryFactor()));
		boundaryFactorTxF.setBounds(455, 105, 60, 20);
		
		// textTxA
		textTxA.setName("textTxA");
		textTxA.setEditable(false);
		textTxA.setText(infoLayout);
		textTxA.setLineWrap(true);
		textTxA.setWrapStyleWord(true);

		Container cp = getContentPane();
		cp.add(diagViewScrP);
		cp.add(optionPnl);
		cp.add(textScrP);

		SpringLayout layout = new SpringLayout();
		layout.putConstraint(SpringLayout.EAST, diagViewScrP, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.WEST, diagViewScrP, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.SOUTH, cp, 130, SpringLayout.SOUTH, diagViewScrP);
		layout.putConstraint(SpringLayout.SOUTH, optionPnl, 0, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.NORTH, optionPnl, 0, SpringLayout.SOUTH, diagViewScrP);
		layout.putConstraint(SpringLayout.EAST, optionPnl, 525, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.WEST, optionPnl, 0, SpringLayout.WEST, cp);

		layout.putConstraint(SpringLayout.EAST, textScrP, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.SOUTH, textScrP, 0, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.WEST, textScrP, 525, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, textScrP, 0, SpringLayout.SOUTH, diagViewScrP);

		cp.setLayout(layout);
		
		
		Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(size);

		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});
		
		try 
		{
			diagram.loadFrom("sample.jd");
		}
		catch (FileNotFoundException exp) 
		{
			JOptionPane.showMessageDialog(AnnealLayoutFrame.this, exp.getMessage());
		}
		catch (IOException exp)
		{
			JOptionPane.showMessageDialog(AnnealLayoutFrame.this, exp.getMessage());
		}
	}

	private Diagram diagram;
	private DiagramView diagramView;
	private JScrollPane diagViewScrP;
	private JButton layoutBtn;
	private JButton loadBtn;
	private JButton saveBtn;
	private JButton importBtn;
	private JButton exportBtn;
	private JTextArea textTxA;
	private JScrollPane textScrP;
	
	private JPanel optionPnl;
	private JLabel iterationLb;
	private JLabel stageLb;
	private JLabel initialLb;
	private JLabel tempScaleLb;
	private JLabel arrowLengthFactorLb;
	private JLabel crossingArrowsCostLb;
	private JLabel distributionFactorLb;
	private JLabel nodeArrowDistFactorLb;
	private JLabel boundaryFactorLb;

	private JTextField iterationTxF;
	private JTextField stageTxF;
	private JTextField initialTxF;
	private JTextField tempScaleTxF;
	private JTextField arrowLengthFactorTxF;
	private JTextField crossingArrowsCostTxF;
	private JTextField distributionFactorTxF;
	private JTextField nodeArrowDistFactorTxF;
	private JTextField boundaryFactorTxF;

	private String infoLayout =
		"The AnnealLayout class implements a Simulated Annealing graph layout algorithm " +
		"that can be used to arrange JDiagram diagrams. Simulated Annealing is a general-purpose " +
		"optimization method used to solve large-scale combinatorial problems by simulating " +
		"the process of heating and cooling of metal to achieve freedom from defects. Finding a nice " +
		"arrangement of a graph is a combinatorial problem that can be reduced to assigning costs " +
		"to graph configurations and finding the minimum cost configuration. In that case, a cost is " +
		"assigned to a graph configuration by evaluating different aesthetic criteria " +
		"such as distance between nodes, length of arrows and the number of arrow crossings.";
	
	private static final long serialVersionUID = 1L;
}