import java.awt.*;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.io.*;

import javax.swing.*;
import javax.swing.SpringLayout;
import javax.swing.event.*;

import com.mindfusion.diagramming.*;

public class FormEditorFrame extends JFrame
{
	public FormEditorFrame()
	{
		super("MindFusion.Diagramming Sample: Form Editor");
		initDiagram();
		initFrame();
	}

	private void initDiagram()
	{
		transferHandler = new FormEditorTransferHandler();
		
		// diagram
		diagram = new Diagram();
		diagram.setShowGrid(true);
		diagram.setBackBrush(new SolidBrush(new Color(220, 220, 220)));

		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeCreated(NodeEvent e)
			{
				diagramNodeCreated(e);
			}

			public void initializeNode(NodeEvent e)
			{
				diagramInitializeNode(e);
			}

			public void nodeSelected(NodeEvent e)
			{
				diagramNodeSelected(e);
			}

			public void nodeDeselected(NodeEvent e)
			{
				diagramNodeDeselected();
			}
		});
		
		// diagramView
		diagramView = new DiagramView(diagram);
		diagramView.setBehavior(Behavior.LinkControls);
		diagramView.setTransferHandler(transferHandler);

		diagramView.addMouseMotionListener(new MouseMotionAdapter()
		{
        	public void mouseMoved(MouseEvent e)
        	{
        		if(FormEditorTransferHandler.isBoxCreated())
        		{
        			nodeCreating(e);
        		}
        	}
        });
	}
	
	protected void diagramNodeDeselected()
	{
		propertyGrid.setSelectedObject(null);
	}

	protected void diagramNodeSelected(NodeEvent e)
	{
		selectedDiagramItem(e.getNode());
	}
	
	private void selectedDiagramItem(DiagramItem item)
	{
		if (!(item instanceof ControlNode))
		{
			propertyGrid.setSelectedObject(null);
			return;
		}
		
		ControlNode node = (ControlNode)item;
		propertyGrid.setSelectedObject(node.getControl());
	}

	protected void nodeCreating(MouseEvent e)
	{
		Point boxDevPos = new Point(e.getX(), e.getY());
		Point2D.Float boxDocPos = diagramView.deviceToDoc(boxDevPos);

		JComponent component = new JButton(FormEditorTransferHandler.getText());
		ControlNode node = new ControlNode(diagramView);
		if (controlList.getSelectedIndex() == 1)
		{
			component = new JTextArea(FormEditorTransferHandler.getText());
			node.setMouseInputMode(MouseInputMode.HandledByHostedControl);
		}
		else if (controlList.getSelectedIndex() == 2)
		{
			component = new JLabel(FormEditorTransferHandler.getText());
			((JLabel)component).setHorizontalAlignment(SwingConstants.CENTER);
		}
		
		node.setControl(component);
		node.setBounds(((int)boxDocPos.getX() - 10),  ((int)boxDocPos.getY() - 10), 20, 20);
		diagram.add(node);
		
		FormEditorTransferHandler.cleanText();
		FormEditorTransferHandler.setBoxCreate(false);
	}

	protected void diagramInitializeNode(NodeEvent e)
	{
		if (!(e.getNode() instanceof ControlNode))
			return;
		
		ControlNode node = (ControlNode)e.getNode();

		if (node.getControl() instanceof JLabel)
		{
			JLabel label = (JLabel)node.getControl();
			label.setHorizontalAlignment(SwingConstants.CENTER);
			label.setText("JLabel");
		}
		if (node.getControl() instanceof JTextArea)
			node.setMouseInputMode(MouseInputMode.HandledByHostedControl);
	}

	protected void diagramNodeCreated(NodeEvent e)
	{
		if (e.getNode() instanceof ControlNode)
		{
			ControlNode node = (ControlNode)e.getNode();
			if (node.getControl() instanceof JButton)
			{
				JButton button = (JButton)node.getControl();
				button.setText("Button" + diagram.getNodes().size());
			}
		}
	}

	protected void listSelectionChanged(ListSelectionEvent arg0)
		throws Exception
	{
		switch (controlList.getSelectedIndex())
		{
			case 0:
			{
				diagram.setDefaultControlType(JButton.class);
				break;
			}
			case 1:
			{
				diagram.setDefaultControlType(JTextArea.class);
				break;
			}
			case 2:
			{
				diagram.setDefaultControlType(JLabel.class);
				break;
			}
		}
	}

	protected void loadDiagram()
	{
		if( fileDialog.showOpenDialog(FormEditorFrame.this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				diagram.loadFrom(fileDialog.getSelectedFile().getAbsolutePath());
				selectedDiagramItem(diagram.getActiveItem());
			}
			catch(Exception exp){
				exp.getMessage();
			}
		}
	}

	protected void saveDiagram()
	{
		if(fileDialog.showSaveDialog(FormEditorFrame.this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				diagram.saveTo(fileDialog.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException exp)
			{
				exp.getMessage();
			}
			catch (IOException exp)
			{
				exp.getMessage();
			}
		}
	}
	
	private void initFrame()
	{
		fileDialog = new JFileChooser();
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// scrollPane
		JScrollPane scrollPane = new JScrollPane(diagramView);
		
		// right panel
		JPanel rightPanel = new JPanel();
		JPanel rightPanelTop = new JPanel();
		rightPanelTop.setLayout(null);
		
		JPanel rightPanelBottom = new JPanel();
		
		// list box
		controlList = new JList(new String[]{"JButton", "JTextArea", "JLabel"});
		controlList.setBounds(10, 10, 130, 60);
		controlList.setDragEnabled(true);
		controlList.setTransferHandler(transferHandler);
		controlList.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent arg0)
			{
				try
				{
					listSelectionChanged(arg0);
				}
				catch (Exception exp)
				{
				}
			}
		});
		
		JButton btnSave = new JButton("Save");
		btnSave.setBounds(150, 10, 130, 25);
		btnSave.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				saveDiagram();
			}
		});
		
		JButton btnLoad = new JButton("Load");
		btnLoad.setBounds(150, 45, 130, 25);
		btnLoad.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent arg0)
			{
				loadDiagram();
			}
		});
		
		// add jlist and two buttons to the right-top panel
		rightPanelTop.add(controlList);
		rightPanelTop.add(btnSave);
		rightPanelTop.add(btnLoad);
		
		// arrange the children of the right panel
		SpringLayout layout = new SpringLayout();
		layout.putConstraint(SpringLayout.EAST, rightPanelTop, 0, SpringLayout.EAST, rightPanel);
		layout.putConstraint(SpringLayout.NORTH, rightPanelTop, 0, SpringLayout.NORTH, rightPanel);
		layout.putConstraint(SpringLayout.WEST, rightPanelTop, 0, SpringLayout.WEST, rightPanel);
		layout.putConstraint(SpringLayout.SOUTH, rightPanelTop, 80, SpringLayout.NORTH, rightPanel);
		
		layout.putConstraint(SpringLayout.EAST, rightPanelBottom, 0, SpringLayout.EAST, rightPanel);
		layout.putConstraint(SpringLayout.NORTH, rightPanelBottom, 0, SpringLayout.SOUTH, rightPanelTop);
		layout.putConstraint(SpringLayout.WEST, rightPanelBottom, 0, SpringLayout.WEST, rightPanel);
		layout.putConstraint(SpringLayout.SOUTH, rightPanelBottom, 0, SpringLayout.SOUTH, rightPanel);
		
		rightPanel.add(rightPanelTop);
		rightPanel.add(rightPanelBottom);
		rightPanel.setLayout(layout);
		
		propertyGrid = new PropertyGrid();
		
		GridLayout gLayout = new GridLayout(1, 1);
		rightPanelBottom.setLayout(gLayout);
		rightPanelBottom.setBackground(Color.blue);
		rightPanelBottom.add(new JScrollPane(propertyGrid));
		
		// layout the scrollpane and the right panel
		Container cp = getContentPane();
		
		layout = new SpringLayout();
		layout.putConstraint(SpringLayout.EAST, scrollPane, -300, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.NORTH, scrollPane, 0, SpringLayout.NORTH, cp);
		layout.putConstraint(SpringLayout.WEST, scrollPane, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.SOUTH, scrollPane, 0, SpringLayout.SOUTH, cp);
		
		layout.putConstraint(SpringLayout.EAST, rightPanel, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.NORTH, rightPanel, 0, SpringLayout.NORTH, cp);
		layout.putConstraint(SpringLayout.WEST, rightPanel, 0, SpringLayout.EAST, scrollPane);
		layout.putConstraint(SpringLayout.SOUTH, rightPanel, 0, SpringLayout.SOUTH, cp);
		
		cp.add(scrollPane);
		cp.add(rightPanel);
		
		cp.setLayout(layout);
		
		setSize(600, 550);
		
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});
	}

	private JList controlList;
	private Diagram diagram;
	private DiagramView diagramView;
	private JFileChooser fileDialog;
	private FormEditorTransferHandler transferHandler;
	private PropertyGrid propertyGrid;
	
	private static final long serialVersionUID = 1L;
}