import java.awt.Color;
import java.awt.Container;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Random;

import javax.swing.*;
import javax.swing.SpringLayout;

import com.mindfusion.diagramming.*;


public class MainFrame extends JFrame
{
	public MainFrame()
	{
		super("MindFusion.Diagramming sample: Layered Layout");
		diagram = new Diagram();
		diagram.setBackBrush(new SolidBrush(Color.white));
		diagram.getShapeNodeStyle().setBrush(new GradientBrush(
				new Color(111, 127, 159), new Color(207, 223, 255), 30));
		
		diagramView = new DiagramView(diagram);
		scrPane = new JScrollPane(diagramView);
		
		btnLayout = new JButton("Random graph");
		btnLayout.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{
				arrangeDiagram();
			}
		});
		
		SpringLayout layout = new SpringLayout();
		JLayeredPane lp = new JLayeredPane();
		layout.putConstraint(SpringLayout.EAST, scrPane, 0, SpringLayout.EAST, lp);
		layout.putConstraint(SpringLayout.SOUTH, scrPane, 0, SpringLayout.SOUTH, lp);
		layout.putConstraint(SpringLayout.WEST, scrPane, 0, SpringLayout.WEST, lp);
		layout.putConstraint(SpringLayout.NORTH, scrPane, 0, SpringLayout.NORTH, lp);

		layout.putConstraint(SpringLayout.EAST, btnLayout, -25, SpringLayout.EAST, lp);
		layout.putConstraint(SpringLayout.NORTH, btnLayout, 10, SpringLayout.NORTH, lp);
		
		lp.add(scrPane, new Integer(0));
		lp.add(btnLayout, new Integer(1));
		lp.setLayout(layout);
		this.setLayeredPane(lp);

		this.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		this.setVisible(true);
		
		this.addWindowListener(new WindowAdapter(){
			  public void windowClosing(WindowEvent arg0)
			  {
				  System.exit(0);
			  }
		});
	}
	
	private void arrangeDiagram()
	{
		randomGraph();
        arrange();
	}
	
	private void randomGraph()
	{
		diagram.clearAll();

        for (int i = 0; i < 30; ++i)
        {
            int c = diagram.getNodes().size();
            int g = 2 + random.nextInt(15);
            for (int j = 0; j < g; ++j)
            {
                ShapeNode node = diagram.getFactory().createShapeNode(0, 0, 40, 40);
                node.setAnchorPattern(AnchorPattern.fromId("TopInBottomOut"));
                if (j > 0)
                    diagram.getFactory().createDiagramLink(diagram.getNodes().get(diagram.getNodes().size() - 2), node);
            }
            if (i > 0)
            {
                for (int j = 0; j < 1 + random.nextInt(3); ++j)
                    diagram.getFactory().createDiagramLink(
                        diagram.getNodes().get(random.nextInt(c)),
                        diagram.getNodes().get(c + random.nextInt(g)));
            }
        }
	}
	
    private void arrange()
    {
    	LayeredLayout layout = new LayeredLayout();
        layout.setAnchoring(Anchoring.Reassign);
        layout.setEnforceLinkFlow(true);
        layout.setStraightenLongLinks(true);
        layout.setNodeDistance(10);
        layout.setLayerDistance(15);
        layout.arrange(diagram);

        diagram.resizeToFitItems(5);
        diagramView.zoomToFit(diagram.getBounds());
    }
	
	private Random random = new Random();
	
	private Diagram diagram;
	private DiagramView diagramView;
	private JScrollPane scrPane;
	private JButton btnLayout;
}
