/**
 * Copyright (c) 2013, MindFusion LLC - Bulgaria.
 */

package samples.tabbedview;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.ItemEvent;
import java.awt.geom.*;
import java.io.*;
import java.util.*;

import javax.imageio.*;
import javax.swing.*;
import javax.swing.filechooser.*;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class MainFrame extends JFrame
	implements ActionListener, ItemListener, TabbedScrollPaneListener
{
	public MainFrame()
	{
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationByPlatform(true);
		setSize(800, 600);
		setTitle("JDiagram Sample: Magnifier");
		try
		{
			ArrayList<Image> images = new ArrayList<Image>();
			images.add(ImageIO.read(App.class.getResourceAsStream("App.png")));
			images.add(ImageIO.read(App.class.getResourceAsStream("App32.png")));
			setIconImages(images);
		}
		catch (Exception ex)
		{
		}

		initializeComponent();

		DiagramStyle style = new DiagramStyle();
		style.setBackBrush(new SolidBrush(Colors.White));
		style.setShadowBrush(new SolidBrush(new Color(0, 0, 0, 20)));

		ShapeNodeStyle nodeStyle = new ShapeNodeStyle();
		nodeStyle.setBrush(new GradientBrush(Colors.White, new Color(219, 231, 240), 90));

		DiagramLinkStyle linkStyle = new DiagramLinkStyle();
		linkStyle.setBrush(new SolidBrush(Colors.Black));

		theme = new Theme();
		theme.registerStyle(Diagram.class, style);
		theme.registerStyle(ShapeNode.class, nodeStyle);
		theme.registerStyle(DiagramLink.class, linkStyle);

		linkShape = new Shape(
			new ElementTemplate[]
			{
				new LineTemplate(20, 100, 50, 0),
				new LineTemplate(50, 0, 80, 100),
				new LineTemplate(80, 100, 20, 100)
			},
			GeneralPath.WIND_EVEN_ODD, "myShape");

		setupPage(documentView.getCurrentPage());
	}

	private void initializeComponent()
	{
		setLayout(new BorderLayout());

		menu = new JMenuBar();
		fileMenu = new JMenu("File");
		openMenuItem = new JMenuItem("Open");
		saveMenuItem = new JMenuItem("Save");
		previewMenuItem = new JMenuItem("Print Preview");
		printMenuItem = new JMenuItem("Print");
		exitMenuItem = new JMenuItem("Exit");
		panel1 = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 5, 0));
		groupBoxShowTabs = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 0, 0));
		radioNever = new JRadioButton("Never");
		radioLeft = new JRadioButton("Left-aligned");
		radioTop = new JRadioButton("Top-aligned");
		radioRight = new JRadioButton("Right-aligned");
		radioBottom = new JRadioButton("Bottom-aligned");
		showTabsGroup = new ButtonGroup();
		groupBoxOptions = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 0, 0));
		checkShowCreateNewButton = new JCheckBox("Show 'Create new' button");
		checkShowCloseButtons = new JCheckBox("Show close buttons");
		checkShowNavigationButtons = new JCheckBox("Show navigation buttons");
		overview = new Overview();
		documentView = new DocumentView();
		tabbedScrollPane = new TabbedScrollPane(documentView);
		// fileMenu
		fileMenu.setMnemonic('f');
		fileMenu.add(openMenuItem);
		fileMenu.add(saveMenuItem);
		fileMenu.add(new JSeparator());
		fileMenu.add(previewMenuItem);
		fileMenu.add(printMenuItem);
		fileMenu.add(new JSeparator());
		fileMenu.add(exitMenuItem);
		// openMenuItem
		openMenuItem.setMnemonic('o');
		openMenuItem.setAccelerator(KeyStroke.getKeyStroke(Character.valueOf('O'), InputEvent.CTRL_DOWN_MASK));
		openMenuItem.addActionListener(this);
		// saveMenuItem
		saveMenuItem.setMnemonic('s');
		saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(Character.valueOf('S'), InputEvent.CTRL_DOWN_MASK));
		saveMenuItem.addActionListener(this);
		// previewMenuItem
		previewMenuItem.setMnemonic('v');
		previewMenuItem.addActionListener(this);
		// printMenuItem
		printMenuItem.setMnemonic('p');
		printMenuItem.setAccelerator(KeyStroke.getKeyStroke(Character.valueOf('P'), InputEvent.CTRL_DOWN_MASK));
		printMenuItem.addActionListener(this);
		// exitMenuItem
		exitMenuItem.setMnemonic('x');
		exitMenuItem.addActionListener(this);
		// menu
		menu.add(fileMenu);
		setJMenuBar(menu);
		// panel1
		panel1.setPreferredSize(new Dimension(200, 538));
		panel1.add(Box.createVerticalStrut(10));
		panel1.add(groupBoxShowTabs, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(groupBoxOptions, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(new JLabel("Overview:"));
		panel1.add(Box.createVerticalStrut(2));
		panel1.add(overview);
		// groupBoxShowTabs
		groupBoxShowTabs.setBorder(BorderFactory.createTitledBorder("Show Tabs"));
		groupBoxShowTabs.add(radioNever);
		groupBoxShowTabs.add(radioLeft);
		groupBoxShowTabs.add(radioTop);
		groupBoxShowTabs.add(radioRight);
		groupBoxShowTabs.add(radioBottom);
		// radioNever
		radioNever.setSelected(true);
		radioNever.setActionCommand("showTabs");
		radioNever.addActionListener(this);
		// radioLeft
		radioLeft.setActionCommand("showTabs");
		radioLeft.addActionListener(this);
		// radioTop
		radioTop.setActionCommand("showTabs");
		radioTop.addActionListener(this);
		// radioRight
		radioRight.setActionCommand("showTabs");
		radioRight.addActionListener(this);
		// radioBottom
		radioBottom.setActionCommand("showTabs");
		radioBottom.addActionListener(this);
		// showTabsGroup
		showTabsGroup.add(radioNever);
		showTabsGroup.add(radioLeft);
		showTabsGroup.add(radioTop);
		showTabsGroup.add(radioRight);
		showTabsGroup.add(radioBottom);
		// groupBoxOptions
		groupBoxOptions.setBorder(BorderFactory.createTitledBorder("Options"));
		groupBoxOptions.add(checkShowCreateNewButton);
		groupBoxOptions.add(checkShowCloseButtons);
		groupBoxOptions.add(checkShowNavigationButtons);
		// checkShowCreateNewButton
		checkShowCreateNewButton.setSelected(true);
		checkShowCreateNewButton.addItemListener(this);
		// checkShowCloseButtons
		checkShowCloseButtons.setSelected(true);
		checkShowCloseButtons.addItemListener(this);
		// checkShowNavigationButtons
		checkShowNavigationButtons.setSelected(true);
		checkShowNavigationButtons.addItemListener(this);
		// overview
		overview.setDiagramView(documentView);
		overview.setFitAll(true);
		overview.setPreferredSize(new Dimension(190, 190));
		overview.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		// tabbedScrollPane
		tabbedScrollPane.addTabbedScrollPaneListener(this);

		Container c = getContentPane();
		c.add(panel1, BorderLayout.EAST);
		c.add(tabbedScrollPane, BorderLayout.CENTER);
	}

	/**
	 * {@link ActionListener#actionPerformed} implementation.
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == openMenuItem)
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
			if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				try
				{
					documentView.loadFromXml(chooser.getSelectedFile().getAbsolutePath());
				}
				catch (XmlException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to load: " + ex.getMessage(),
						"Open document", JOptionPane.ERROR_MESSAGE);
				}
				catch (IOException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to load: " + ex.getMessage(),
						"Open document", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		else if (e.getSource() == saveMenuItem)
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
			if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				try
				{
					documentView.saveToXml(chooser.getSelectedFile().getAbsolutePath());
				}
				catch (XmlException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to save: " + ex.getMessage(),
						"Save document", JOptionPane.ERROR_MESSAGE);
				}
				catch (IOException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to save: " + ex.getMessage(),
						"Save document", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		else if (e.getSource() == previewMenuItem)
		{
			documentView.printPreview();
		}
		else if (e.getSource() == printMenuItem)
		{
			documentView.print();
		}
		else if (e.getSource() == exitMenuItem)
		{
			WindowEvent event = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
			Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(event);
		}
		else if ("showTabs".equals(e.getActionCommand()))
		{
			tabbedScrollPane.setShowTabs(!radioNever.isSelected());
			if (!radioNever.isSelected())
			{
				if (radioLeft.isSelected())
					tabbedScrollPane.setTabAlignment(JTabbedPane.LEFT);
				else if (radioTop.isSelected())
					tabbedScrollPane.setTabAlignment(JTabbedPane.TOP);
				else if (radioRight.isSelected())
					tabbedScrollPane.setTabAlignment(JTabbedPane.RIGHT);
				else if (radioBottom.isSelected())
					tabbedScrollPane.setTabAlignment(JTabbedPane.BOTTOM);
			}
		}
	}

	/**
	 * {@link ItemListener#itemStateChanged} implementation.
	 */
	public void itemStateChanged(ItemEvent e)
	{
		if (e.getSource() == checkShowCreateNewButton)
		{
			tabbedScrollPane.setShowCreateNewTab(checkShowCreateNewButton.isSelected());
		}
		else if (e.getSource() == checkShowCloseButtons)
		{
			tabbedScrollPane.setShowTabCloseButtons(checkShowCloseButtons.isSelected());
		}
		else if (e.getSource() == checkShowNavigationButtons)
		{
			tabbedScrollPane.setShowTabNavigationButtons(checkShowNavigationButtons.isSelected());
		}
	}

	/**
	 * {@link TabbedScrollPaneListener#currentPageChanged} implementation.
	 */
	public void currentPageChanged(EventObject e)
	{
		setupPage(tabbedScrollPane.getCurrentPage());
		overview.repaint();
	}

	/**
	 * {@link TabbedScrollPaneListener#pageAdded} implementation.
	 */
	public void pageAdded(DiagramPageEvent e)
	{
	}

	/**
	 * {@link TabbedScrollPaneListener#pageRemoved} implementation.
	 */
	public void pageRemoved(DiagramPageEvent e)
	{
	}

	/**
	 * {@link TabbedScrollPaneListener#pageRemoving} implementation.
	 */
	public void pageRemoving(DiagramPageValidationEvent e)
	{
		if (documentView.getDocument().getPages().size() == 1)
		{
			JOptionPane.showMessageDialog(this, "The last page cannot be removed.",
				"Removing page", JOptionPane.INFORMATION_MESSAGE);
			e.setCancel(true);
		}
	}

	private void setupPage(DiagramPage page)
	{
		if (page == null)
			return;

		page.setTheme(theme);
		page.setShowGrid(true);
		page.setGridStyle(GridStyle.Points);
		page.setDefaultShape(Shape.fromId("Rectangle"));
		page.setRouteLinks(true);
		page.setLinkHeadShape(linkShape);
		page.setLinkHeadShapeSize(3);
		page.setShadowOffsetX(1);
		page.setShadowOffsetY(1);
	}


	private static final long serialVersionUID = 1L;

	private TabbedScrollPane tabbedScrollPane;
	private DocumentView documentView;
	private JMenuBar menu;
	private JMenu fileMenu;
	private JMenuItem openMenuItem;
	private JMenuItem saveMenuItem;
	private JMenuItem previewMenuItem;
	private JMenuItem printMenuItem;
	private JMenuItem exitMenuItem;
	private JPanel panel1;
	private JPanel groupBoxShowTabs;
	private JRadioButton radioNever;
	private JRadioButton radioLeft;
	private JRadioButton radioTop;
	private JRadioButton radioRight;
	private JRadioButton radioBottom;
	private ButtonGroup showTabsGroup;
	private JPanel groupBoxOptions;
	private JCheckBox checkShowCreateNewButton;
	private JCheckBox checkShowCloseButtons;
	private JCheckBox checkShowNavigationButtons;
	private Overview overview;

	private Theme theme;
	private Shape linkShape;
}

final class Colors
{
	public static final Color White = new Color(255, 255, 255);
	public static final Color Black = new Color(0, 0, 0);
}
